/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	Cancer class


*******************************************************************************
******************************************************************************/

#include "Blob.h"

#define MIN_X ( -20 << 12 )
#define MIN_Y ( -18 << 12 )

#define MAX_BLOBS 16

//------------------------------------------------------------------------------
//	Blob Enemy
//------------------------------------------------------------------------------
Blob::Blob()
{
	Frame = 0;
	Active = false;	
	DrawMode = EN_NORMAL;
	
}

Blob::~Blob()
{
}


void Blob::Update()
{
	Sx += Dx;
	x = Sx + sinLerp( Counter * 230 ) * Radius;
	if( x < MIN_X )
		Active = false;

	Counter++;
	// animate
	if( ( Counter & 3 ) == 0 )
	{
		Frame = (Frame + 1) & 3;
	}

}


//------------------------------------------------------------------------------
//	Blob Container
//------------------------------------------------------------------------------
//--------------------------------------
//	Ctor and Dtor
//--------------------------------------
BlobContainer::BlobContainer( int Palette, glImage* const Sprites )
{
	ColorTable = Palette;		// init needed data for drawing
	Images = Sprites;
	Blobs.resize( MAX_BLOBS/2 );			// 1/2 of max (speed of iteration)
	Blobs.reserve( MAX_BLOBS );		// reserve max for speed.  No need to do this for enemies
}
	
BlobContainer::~BlobContainer()
{
	Blobs.clear();				// clear the bullets
}

//--------------------------------------
//	Virtual Functions
//--------------------------------------
void BlobContainer::UpdateEntities()
{
	
	std::vector<Blob>::iterator iter;
	for( iter = Blobs.begin(); iter != Blobs.end(); ++iter )
	{
		if( iter->Active )
			iter->Update();
	}
	
}

void BlobContainer::DrawEntities()
{
	
	glColorTable( GL_RGB16, ColorTable );
	numEntities = 0;
	std::vector<Blob>::iterator iter;
	for( iter = Blobs.begin(); iter != Blobs.end(); ++iter )
	{
		if( iter->Active )
		{
			numEntities++;
			if( (iter->DrawMode == EN_WHITE) )
			{
				iter->DrawMode = EN_NORMAL;
				glColorTable( GL_RGB256, Palettes::Instance()->Green() );
				glSprite( (iter->x >> 12) - 19, (iter->y >> 12) - 18, 
					      GL_FLIP_NONE, &Images[iter->Frame] 
					    );
				glColorTable( GL_RGB16, ColorTable );
	
			}
			else
			{
				glSprite( (iter->x >> 12) - 19, (iter->y >> 12) - 18, 
						  GL_FLIP_NONE, &Images[iter->Frame] 
					    );
			}
			
		}
	}
	
}

bool BlobContainer::Collide( StarFighter &Star )
{
	std::vector<Blob>::iterator iter;
	for( iter = Blobs.begin(); iter != Blobs.end(); ++iter )
	{
		if( iter->Active )
		{
			int rad2 = 16 * 16;
			int cx = (iter->x >> 12) - (Star.x >> 12);
			int cy = (iter->y >> 12) - (Star.y >> 12);
			if( ( (cx * cx) + (cy * cy) ) < (rad2 * (1+Star.IsBursting())) )
			{
				if( Star.IsBursting() && (iter->DrawMode == EN_NORMAL) )
				{
					if( (iter->Counter & 1) ) iter->DrawMode = EN_WHITE;
				}
				return true;
			}
		}
	}
	return false;
}


//--------------------------------------
//	Non-Virtual Functions
//--------------------------------------
void BlobContainer::Spawn( int _x, int _y, s32 speed, int _radius )
{
	
	bool SlotFull = true;      		// value to check if all the slots are occupied
	
	std::vector<Blob>::iterator iter;
	for( iter = Blobs.begin(); iter != Blobs.end(); ++iter )
	{
		if( !iter->Active )		// insert a bullet on a free slot
		{

			iter->Sx = _x << 12;
			iter->y = _y << 12;
			iter->Dx = speed;
			iter->Dy = 0;
			iter->Active = true;
			iter->Frame = 0;
			iter->Counter = 0;
			iter->Radius = _radius;
			SlotFull = false;			// there's a free slot so we dont execute the code below
			break;
		
		}
	}
	
	// All the slots are full
	// Only push back a bullet if the slot is full and we have less than numbullets
	if( SlotFull && (Blobs.size() < MAX_BLOBS) )
	{
		Blob Blob;
		Blob.Sx = _x << 12;
		Blob.y = _y << 12;
		Blob.Dx = speed;
		Blob.Dy = 0;
		Blob.Active = true;
		Blob.Frame = 0;
		Blob.Counter = 0;
		Blob.Radius = _radius;
		
		Blobs.push_back(Blob);
	}
	
}

