/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	Centipede class


*******************************************************************************
******************************************************************************/

#include "Centipede.h"

#define MIN_X ( -290 << 12 )
#define MIN_Y ( -18 << 12 )

#define MAX_CENTIPEDES 4


//------------------------------------------------------------------------------
//	Centipede Enemy
//------------------------------------------------------------------------------
Centipede::Centipede()
{
	Index = 0;
	Frame = 0;
	x = floattof32(300);
	y = floattof32(96);
	Active = false;
	Counter = 0;
	DrawMode = EN_NORMAL;
	
	Path = centipede_points;
	
	for( int i = 0; i < CENTIPEDE_MAX_SEGMENTS; i++ )
	{
		Segments[i].x = floattof32(300);
		Segments[i].y = floattof32(96);
	}

}

Centipede::~Centipede()
{
}


void Centipede::Update()
{
	// Update spline

	
	Counter++;
	
	ox = x;
	oy = y;
	
	if( ( Counter & 1 ) == 0 )
	{
		Index++;
		if( Index >= CENTIPEDE_NUM_ELEMENTS ) Index = 0;		
	
		x = Path[Index].x;
		y = Path[Index].y;
		Segments[CENTIPEDE_MAX_SEGMENTS - 1].x = x;
		Segments[CENTIPEDE_MAX_SEGMENTS - 1].y = y;

		//for( int i = (CENTIPEDE_MAX_SEGMENTS - 1); i > 0; i-- )
		for( int i = 0; i < (CENTIPEDE_MAX_SEGMENTS - 1);  i++ )
		{
			Segments[i].x = Segments[i+1].x;
			Segments[i].y = Segments[i+1].y;
		}
		
	}
	
	
	
	if( x < MIN_X )
		Active = false;

	
}


//--------------------------------------
//	Private Functions
//--------------------------------------


//------------------------------------------------------------------------------
//	Centipede Container
//------------------------------------------------------------------------------
//--------------------------------------
//	Ctor and Dtor
//--------------------------------------
CentipedeContainer::CentipedeContainer( int Palette, glImage* const Sprites )
{
	ColorTable = Palette;		// init needed data for drawing
	Images = Sprites;
	Centipedes.resize( MAX_CENTIPEDES/2 );			// 1/2 of max (speed of iteration)
	Centipedes.reserve( MAX_CENTIPEDES );		// reserve max for speed.  No need to do this for enemies
}
	
CentipedeContainer::~CentipedeContainer()
{
	Centipedes.clear();				// clear the bullets
}

//--------------------------------------
//	Virtual Functions
//--------------------------------------
void CentipedeContainer::UpdateEntities()
{
	
	std::vector<Centipede>::iterator iter;
	for( iter = Centipedes.begin(); iter != Centipedes.end(); ++iter )
	{
		if( iter->Active )
			iter->Update();
	}
	
}

void CentipedeContainer::DrawEntities()
{
	
	glColorTable( GL_RGB256, ColorTable );
	numEntities = 0;
	std::vector<Centipede>::iterator iter;
	for( iter = Centipedes.begin(); iter != Centipedes.end(); ++iter )
	{
		if( iter->Active )
		{
			numEntities++;
			if( (iter->DrawMode == EN_WHITE) )
			{
				iter->DrawMode = EN_NORMAL;
				glColorTable( GL_RGB256, Palettes::Instance()->Red() );
			
				for( int i = 0; i < CENTIPEDE_MAX_SEGMENTS; i+= 8 )
				{
					
					s32 tx = iter->Segments[i].x;
					s32 ty = iter->Segments[i].y;
					
					s32 angle;
					if( i == 0 )    //tail
					{
						angle = BRAD_PI + atan2Lerp( iter->Segments[i+1].x - tx, iter->Segments[i+1].y - ty );				
						iter->Frame = 3;					
					}	
					else if( i == (CENTIPEDE_MAX_SEGMENTS-8) )	// head
					{
						angle = BRAD_PI + atan2Lerp( iter->Segments[i-7].x - tx, iter->Segments[i-7].y - ty );
						iter->Frame = 0;
					}
					else	// segments
					{
						angle = BRAD_PI + atan2Lerp( iter->Segments[i-7].x - tx, iter->Segments[i-7].y - ty );
						iter->Frame = 1;
					}
					
					
					glSpriteRotate( tx >> 12,
									ty >> 12,
									angle,
									GL_FLIP_NONE, &Images[iter->Frame] 
								  );
							
				}  //next
				
				glColorTable( GL_RGB16, ColorTable );
					
			}   
			else
			{
				for( int i = 0; i < CENTIPEDE_MAX_SEGMENTS; i+= 8 )
				{
					
					s32 tx = iter->Segments[i].x;
					s32 ty = iter->Segments[i].y;
					
					s32 angle;
					if( i == 0 )    //tail
					{
						angle = BRAD_PI + atan2Lerp( iter->Segments[i+1].x - tx, iter->Segments[i+1].y - ty );				
						iter->Frame = 3;					
					}	
					else if( i == (CENTIPEDE_MAX_SEGMENTS-8) )	// head
					{
						angle = BRAD_PI + atan2Lerp( iter->Segments[i-7].x - tx, iter->Segments[i-7].y - ty );
						iter->Frame = 0;
					}
					else	// segments
					{
						angle = BRAD_PI + atan2Lerp( iter->Segments[i-7].x - tx, iter->Segments[i-7].y - ty );
						iter->Frame = 1;
					}
					
					
					glSpriteRotate( tx >> 12,
									ty >> 12,
									angle,
									GL_FLIP_NONE, &Images[iter->Frame] 
								  );
							
				}  // next
				
			} // endif color mode
		}
	}
	
}

bool CentipedeContainer::Collide( StarFighter &Star )
{
	std::vector<Centipede>::iterator iter;
	for( iter = Centipedes.begin(); iter != Centipedes.end(); ++iter )
	{
		if( iter->Active )
		{
			for( int i = 0; i < CENTIPEDE_MAX_SEGMENTS; i+= 8 )
			{
				int rad2 = 16 * 16;
				int cx = (iter->Segments[i].x >> 12) - (Star.x >> 12);
				int cy = (iter->Segments[i].y >> 12) - (Star.y >> 12);
				if( ( (cx * cx) + (cy * cy) ) < (rad2 * (1+Star.IsBursting())) )
				{
					if( Star.IsBursting() && (iter->DrawMode == EN_NORMAL) )
					{
						if( (iter->Counter & 1) ) iter->DrawMode = EN_WHITE;
					}
					return true;
				}	
			}
			
		}
	}
	
	return false;
}


//--------------------------------------
//	Non-Virtual Functions
//--------------------------------------
void CentipedeContainer::Spawn( int _x, int _y )
{
	bool SlotFull = true;      		// value to check if all the slots are occupied
	
	std::vector<Centipede>::iterator iter;
	for( iter = Centipedes.begin(); iter != Centipedes.end(); ++iter )
	{
		if( !iter->Active )		// insert a bullet on a free slot
		{
			iter->x = _x;
			iter->y = _y;
			iter->Active = true;
			iter->Frame = 0;
			iter->Counter = 0;
			iter->Index = 0;			
			SlotFull = false;			// there's a free slot so we dont execute the code below
			break;
		
		}
	}
	
	// All the slots are full
	// Only push back a bullet if the slot is full and we have less than numbullets
	if( SlotFull && (Centipedes.size() < MAX_CENTIPEDES) )
	{
		Centipede Centipede;
		Centipede.x = _x;
		Centipede.y = _y;
		Centipede.Active = true;
		Centipede.Frame = 0;
		Centipede.Counter = 0;
		Centipede.Index = 0;
		Centipedes.push_back(Centipede);
	}
	
}


	
