/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	Spline class


*******************************************************************************
******************************************************************************/


/*
 *	Fixed Point catmull-rom spline interpolator
 *	relminator
 *	http://rel.betterwebber.com
 *
 */

#include "Spline.h"







/******************************************************************************


******************************************************************************/
/*
 *	params:
 *
 *		P0-Pn -> non fixed point int32 values
 *		T     -> 1.19.12 fixed point int32 value (0-4095)
 *
 *	return: 
 *		-> 1.19.12 fixed point int32 (needs to be >> 12 when used)
 */

s32 catmull_FP_12(int p0, int p1, int p2, int p3, s32 t)
{
	
	s32 t2 = (t * t) >> 12;
	s32 t3 = (t2 * t) >> 12;
		
	s32 f1;
	
	f1 = 	(( 8192 * p1 ) + ( -p0 + p2 ) * t) + 
			( 2 * p0 - 5 * p1 + 4 * p2 - p3 ) * t2 +  		
			( -p0 + 3 * p1 - 3 * p2 + p3 ) * t3; 		
	
	
	
	return (2048 * (f1 >> 12));
	
}


/******************************************************************************


******************************************************************************/
Spline::Spline()
{

	tval = 0;
	speed = 0;
	index = 0;
	max_index = 1;
	
	x = 0;
	y = 0;
}

/******************************************************************************


******************************************************************************/
Spline::~Spline()
{
	
	if (p_coords)
		delete[] p_coords;
	
	p_coords = 0;
}



/******************************************************************************


******************************************************************************/
void Spline::InitCoords(int max_points)
{
	
	if (p_coords)
	{
		delete[] p_coords;
		index = 0;
		max_index = max_points;
		p_coords = new Tcoord[max_index];
	}
	else
	{
		index = 0;
		max_index = max_points;
		p_coords = new Tcoord[max_index];
		
	}

}

/******************************************************************************


******************************************************************************/
void Spline::Update()
{

	Tcoord p0,p1,p2,p3;
	
	u16 i1 = mod32(index+1, max_index);
	u16 i2 = mod32(index+2, max_index);
	u16 i3 = mod32(index+3, max_index);
	
	
	p0.x = p_coords[index].x;
	p0.y = p_coords[index].y;
	p1.x = p_coords[i1].x;
	p1.y = p_coords[i1].y;
	p2.x = p_coords[i2].x;
	p2.y = p_coords[i2].y;
	p3.x = p_coords[i3].x;
	p3.y = p_coords[i3].y;  
	
	x = catmull_FP_12(p0.x,p1.x,p2.x,p3.x,tval);
	y = catmull_FP_12(p0.y,p1.y,p2.y,p3.y,tval);
	
	tval += speed;
	
	if (tval > 4095)
	{
		tval = tval & 4095;
		index = mod32(index+1, max_index);
	}
	
	
}


