/******************************************************************************
*******************************************************************************

	Bubble Fight EX
	relminator
	http://rel.betterwebber.com


	StarFighter class


*******************************************************************************
******************************************************************************/

#include "StarFighter.h"

#define MIN_X ( 0 )
#define MIN_Y ( 0 )

#define MAX_X ( SCREEN_WIDTH << 12 )
#define MAX_Y ( ( SCREEN_HEIGHT-8 ) << 12 )

static const s32 X_FRICTION = floattof32(0.008);
static const s32 Y_FRICTION = floattof32(0.008);
static const s32 GRAVITY = floattof32(0.012);
static const s32 ACCEL = floattof32(0.02);

//--------------------------------------------------------------------
//    
//--------------------------------------------------------------------
StarFighter::StarFighter()
{
	StartFrame = 6;
	Frame = 0;
	Counter = 0;
	BurstCounter = 0;
	Direction = 1;
	x = 20 << 12;
	y = ( SCREEN_HEIGHT/2 ) << 12;
	Dx = 0;
	Dy = 0;
	FlipMode = GL_FLIP_NONE;
	

}

//--------------------------------------------------------------------
//    
//--------------------------------------------------------------------
StarFighter::~StarFighter()
{

}

//--------------------------------------------------------------------
//    
//--------------------------------------------------------------------
void StarFighter::Update( int Key, int KeyD )
{
	Counter++;
	
	
	// process input
	if( Key & KEY_RIGHT )
	{
		Dx += ACCEL;
		Direction = 1;
	}
	
	if( Key & KEY_LEFT )
	{
		Dx -= ACCEL;
		Direction = -1;
	}
	
	if( Key & KEY_DOWN )
	{
		Dy += GRAVITY;
	}

	if( (Key & KEY_UP) || (Key & KEY_A) )
	{
		Dy -= ( ( ACCEL >> 3 ) * Y_FRICTION );
	}
	
	
	if( (KeyD & KEY_B) )
	{
		if( BurstCounter <= 0 )
		{
			BurstCounter = 16;
			Sound::Instance()->PlaySFX(0);
		}
		
	}
	
	
	FlipMode = GL_FLIP_NONE;
	
	if( BurstCounter <= 0 )
	{
	
		Dx -= ( Dx * X_FRICTION ) >> 12;
		Dy += GRAVITY;
		Dy -= ( Dy * Y_FRICTION ) >> 12;

		x += Dx;
		y += Dy;

		// animate
		StartFrame = 6;
		if( ( Counter & 3 ) == 0 )
		{
			if( Direction > 0 )
				Frame = (Frame + 1) & 7;
			else
				Frame = (Frame - 1) & 7;
		}
		
		Bounce();
	}
	else
	{
		BurstCounter--;
		Dx = 0;
		Dy = 0;
		
		if( Direction < 0 )
		{
			x -= 3 << 12;
			FlipMode = GL_FLIP_H;
		}
		else
		{
			x += 3 << 12;
		}
		
		Limit();
		
		// animate
		StartFrame = 0;
		Frame = (Frame + 1);
		if( Frame > 5) Frame = 0;
	
	}
}

//--------------------------------------------------------------------
//    
//--------------------------------------------------------------------
void StarFighter::Draw( int Palette, glImage* const Sprites ) const
{
	// Center the sprite
	// From now on all the sprites are gonna be centerbased
	// Meaning X and Y are the centers
	glColorTable( GL_RGB256, Palette );
	if( (Counter & 1) && (BurstCounter > 0) )
		glColor( RGB15(qran_range(0, 31), qran_range(0, 31), Counter&31) );
	
	int offx = Sprites[Frame+StartFrame].width/2;
	int offy = Sprites[Frame+StartFrame].height/2;
	glSprite( (x >> 12)-offx, (y >> 12)-offy, FlipMode, &Sprites[Frame+StartFrame] );
	
}


//--------------------------------------
//	Private Functions
//--------------------------------------
void StarFighter::Bounce()
{

	if( (x < MIN_X) )
	{
		Dx = -Dx;
		x = MIN_X;
	}
	if( (x > MAX_X) )
	{
		Dx = -Dx;
		x = MAX_X;
	}
	if( (y < MIN_Y) )
	{
		Dy = -Dy;
		y = MIN_Y;
	}
	if( (y > MAX_Y) )  
	{
		Dy = -Dy;
		y = MAX_Y;
	}

}


//--------------------------------------
//	
//--------------------------------------
void StarFighter::Limit()
{

	if( (x < MIN_X) )
	{
		x = MIN_X;
	}
	if( (x > MAX_X) )
	{
		x = MAX_X;
	}
	if( (y < MIN_Y) )
	{
		y = MIN_Y;
	}
	if( (y > MAX_Y) )  
	{
		y = MAX_Y;
	}

}



